package cn.estsh.i3plus.ext.mes.pcn.apiservice.controller.busi;

import cn.estsh.i3plus.ext.mes.pcn.api.base.IMesProdShiftRecordService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.constant.MesCommonConstant;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesProdShiftRecord;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * @Description: 开关班记录
 * @Author: gsz
 * @Date: 2024/5/25 18:16
 * @Modify:
 */
@RestController
@RequestMapping(MesCommonConstant.MES_YANFEN + "/mesProdShiftRecord")
public class MesProdShiftRecordController   {

    @Autowired
    private IMesProdShiftRecordService mesProdShiftRecordService;

    @GetMapping("/query-pager")
    @ApiOperation(value = "查询开关班记录")
    public ResultBean queryMesProdShiftRecordByPager(MesProdShiftRecord mesProdShiftRecord, Pager pager) {
        try {
            ListPager<MesProdShiftRecord> partInspectionListPager = mesProdShiftRecordService.queryMesProdShiftRecordByPager(mesProdShiftRecord, pager);
            return ResultBean.success("查询成功").setListPager(partInspectionListPager);
        } catch (ImppBusiException imppException) {
            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping("/insert")
    @ApiOperation(value = "保存开关班记录")
    public ResultBean insertMesProdShiftRecord(@RequestBody MesProdShiftRecord item) {

        try {
            // 数据校验
            ValidatorBean.checkNotNull(item.getOrganizeCode(), "工厂号不能为空");
            ValidatorBean.checkNotNull(item.getUserInfo(), "操作人不能为空");
            ValidatorBean.checkNotNull(item.getWorkCenterCode(), "workCenterCode不能为空" );
            ValidatorBean.checkNotNull(item.getShiftGroup(), "shiftGroup不能为空");
            ValidatorBean.checkNotNull(item.getShiftCode(), "shiftCode不能为空");

            mesProdShiftRecordService.insertMesProdShiftRecord(item);

            return ResultBean.success("保存开关班记录成功");
        } catch (ImppBusiException imppException) {
            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping("/update")
    @ApiOperation(value = "保存开关班记录")
    public ResultBean updateMesProdShiftRecord(@RequestBody MesProdShiftRecord item) {

        try {
            // 数据校验
            ValidatorBean.checkNotNull(item.getOrganizeCode(), "工厂号不能为空");
            ValidatorBean.checkNotNull(item.getUserInfo(), "操作人不能为空");
            ValidatorBean.checkNotNull(item.getWorkCenterCode(), "workCenterCode不能为空" );
            ValidatorBean.checkNotNull(item.getShiftGroup(), "shiftGroup不能为空");
            ValidatorBean.checkNotNull(item.getShiftCode(), "shiftCode不能为空");

            mesProdShiftRecordService.saveMesProdShiftRecord(item);

            return ResultBean.success("保存开关班记录成功");
        } catch (ImppBusiException imppException) {
            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/query-prod-shift-record")
    @ApiOperation(value = "查询开关班记录")
    public ResultBean queryMesProdShiftRecord(MesProdShiftRecord mesProdShiftRecord) {
        try {
            //查询汇总工单数量
            MesProdShiftRecord mesPartProdGroup= mesProdShiftRecordService.queryMesProdShiftRecord(mesProdShiftRecord);
            return ResultBean.success("查询成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultObject(mesPartProdGroup);
        } catch (ImppBusiException e) {
            return ResultBean.fail(e).build();
        } catch (Exception e) {
            return ResultBean.fail(e);
        }
    }

}
