package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.pcn.api.base.IMesEquipmentExtService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesCellEquipContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.repository.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@Service
public class MesEquipmentExtService implements IMesEquipmentExtService {

    @Autowired
    private MesWcEquipmentRepository wcEquipmentRepository;

    @Autowired
    private MesEquipmentRepository equipmentRepository;

    @Autowired
    private MesEquipmentVariableRepository equipmentVariableRepository;

    @Autowired
    private MesEquipmentVariableCfgRepository equipmentVariableCfgRepository;

    @Autowired
    private MesEquipmentChannelRepository equipmentChannelRepository;

    @Autowired
    private MesCavityGroupDetailCfgRepository mesCavityGroupDetailCfgRepository;

    @Override
    public List<MesCellEquipContext> getCellEquipmentList(String organizeCode, String workCenterCode, String workCellCode) {
        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(workCenterCode) || StringUtils.isEmpty(workCellCode)) return null;
        List<MesWcEquipment> wcEquipmentDbList = getWcEquipmentList(organizeCode, workCenterCode, workCellCode);
        if (CollectionUtils.isEmpty(wcEquipmentDbList)) return null;
        List<MesCellEquipContext> cellEquipmentContextList = null;
        for (MesWcEquipment wcEquipmentDb : wcEquipmentDbList) {
            if (null == wcEquipmentDb) continue;
            MesEquipment equipmentDb = equipmentRepository.getByProperty(
                    new String[]{MesPcnExtConstWords.ORGANIZE_CODE, MesPcnExtConstWords.IS_DELETED, MesPcnExtConstWords.IS_VALID, MesPcnExtConstWords.EQUIPMENT_CODE},
                    new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), CommonEnumUtil.IS_VAILD.VAILD.getValue(), wcEquipmentDb.getEquipmentCode()});
            if (null == equipmentDb) continue;
            if (CollectionUtils.isEmpty(cellEquipmentContextList)) cellEquipmentContextList = new ArrayList<>();
            MesCellEquipContext cellEquipmentContext = new MesCellEquipContext();
            BeanUtils.copyProperties(wcEquipmentDb, cellEquipmentContext);
            BeanUtils.copyProperties(equipmentDb, cellEquipmentContext);
            cellEquipmentContextList.add(cellEquipmentContext);
        }
        return cellEquipmentContextList;
    }

    @Override
    public List<MesWcEquipment> getWcEquipmentList(String organizeCode, String workCenterCode, String workCellCode) {
        return wcEquipmentRepository.findByProperty(
                new String[]{MesPcnExtConstWords.ORGANIZE_CODE, MesPcnExtConstWords.IS_DELETED, MesPcnExtConstWords.IS_VALID, MesPcnExtConstWords.WORK_CENTER_CODE, MesPcnExtConstWords.WORK_CELL_CODE},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), CommonEnumUtil.IS_VAILD.VAILD.getValue(), workCenterCode, workCellCode});
    }

    @Override
    public MesWcEquipment getWcEquipment(String organizeCode, String workCenterCode, String workCellCode) {
        List<MesWcEquipment> wcEquipmentDbList = getWcEquipmentList(organizeCode, workCenterCode, workCellCode);
        if (CollectionUtils.isEmpty(wcEquipmentDbList)) return null;
        if (wcEquipmentDbList.size() == 1) return wcEquipmentDbList.get(0);
        List<MesWcEquipment> wcEquipmentDbListSort = wcEquipmentDbList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getSeq()))).sorted(Comparator.comparing(MesWcEquipment::getSeq)).collect(Collectors.toList());
        return !CollectionUtils.isEmpty(wcEquipmentDbListSort) ? wcEquipmentDbListSort.get(0) : wcEquipmentDbList.get(0);
    }

    @Override
    public List<MesEquipmentVariableCfg> getEquipmentVariableCfgList(String organizeCode, String variableCfgType) {
        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(variableCfgType)) return null;
        return equipmentVariableCfgRepository.findByProperty(
                new String[]{MesPcnExtConstWords.ORGANIZE_CODE, MesPcnExtConstWords.IS_DELETED, MesPcnExtConstWords.IS_VALID, MesPcnExtConstWords.VARIABLE_CFG_TYPE},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), CommonEnumUtil.IS_VAILD.VAILD.getValue(), variableCfgType});
    }

    @Override
    public List<MesEquipmentVariable> getEquipmentVariableList(String organizeCode, String equipmentCode, Integer variableType) {
        if (StringUtils.isEmpty(variableType) || StringUtils.isEmpty(equipmentCode) || StringUtils.isEmpty(variableType)) return null;
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(equipmentCode, MesPcnExtConstWords.EQUIPMENT_CODE, packBean);
        DdlPreparedPack.getNumEqualPack(variableType, MesPcnExtConstWords.VARIABLE_TYPE, packBean);
        return equipmentVariableRepository.findByHqlWhere(packBean);
    }

    @Override
    public List<MesEquipmentChannel> getEquipmentChannelList(String organizeCode, String equipmentCode) {
        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(equipmentCode)) return null;
        return equipmentChannelRepository.findByProperty(
                new String[]{MesPcnExtConstWords.ORGANIZE_CODE, MesPcnExtConstWords.IS_DELETED, MesPcnExtConstWords.IS_VALID, MesPcnExtConstWords.EQUIPMENT_CODE},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), CommonEnumUtil.IS_VAILD.VAILD.getValue(), equipmentCode});
    }

    @Override
    public List<MesCavityGroupDetailCfg> getMesCavityGroupDetailCfgList(String organizeCode, String equipmentCode) {

        //获取腔组数据信息集合
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(equipmentCode, "equipmentCode", packBean);
        return mesCavityGroupDetailCfgRepository.findByHqlWhere(packBean);
    }

    @Override
    public MesEquipment getMesEquipment(String organizeCode, String equipmentCode) {
        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(equipmentCode)) return null;
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(equipmentCode, MesPcnExtConstWords.EQUIPMENT_CODE, packBean);
        return equipmentRepository.getByProperty(packBean);
    }

    @Override
    public List<MesEquipmentVariable> getEquipmentVariableList(String organizeCode, List<Long> equipVariableIdList) {
        if (StringUtils.isEmpty(organizeCode) || CollectionUtils.isEmpty(equipVariableIdList)) return null;
        equipVariableIdList = equipVariableIdList.stream().filter(o -> !StringUtils.isEmpty(o)).distinct().collect(Collectors.toList());
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        if (equipVariableIdList.size() == 1) DdlPreparedPack.getNumEqualPack(equipVariableIdList.get(0), MesPcnExtConstWords.ID, packBean);
        else DdlPreparedPack.getInPackList(equipVariableIdList, MesPcnExtConstWords.ID, packBean);
        return equipmentVariableRepository.findByHqlWhere(packBean);
    }

    @Override
    public MesEquipmentVariable getMesEquipmentVariable(String organizeCode, String equipmentCode, long equipmentVariableId) {
        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(equipmentCode) || equipmentVariableId == 0) return null;
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(equipmentCode, MesPcnExtConstWords.EQUIPMENT_CODE, packBean);
        DdlPreparedPack.getNumEqualPack(equipmentVariableId, MesPcnExtConstWords.ID, packBean);
        return equipmentVariableRepository.getByProperty(packBean);
    }

    @Override
    public MesEquipmentChannel getMesEquipmentChannel(String organizeCode, String equipmentCode, String channel) {
        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(equipmentCode) || StringUtils.isEmpty(channel)) return null;
        return equipmentChannelRepository.getByProperty(
                new String[]{MesPcnExtConstWords.ORGANIZE_CODE, MesPcnExtConstWords.IS_DELETED, MesPcnExtConstWords.IS_VALID, MesPcnExtConstWords.EQUIPMENT_CODE, MesPcnExtConstWords.CHANNEL},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), CommonEnumUtil.IS_VAILD.VAILD.getValue(), equipmentCode, channel});
    }

    @Override
    public MesEquipment getMesEquipmentByEquipId(String organizeCode, Integer equipId) {
        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(equipId)) return null;
        return equipmentRepository.getByProperty(
                new String[]{MesPcnExtConstWords.ORGANIZE_CODE, MesPcnExtConstWords.IS_DELETED, MesPcnExtConstWords.IS_VALID, MesPcnExtConstWords.EQUIP_ID},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), CommonEnumUtil.IS_VAILD.VAILD.getValue(), equipId});
    }
}
