package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.pcn.api.base.IMesTemplateService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.util.MesPcnException;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.platform.common.util.MesPcnConstWords;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesLabelTemplate;
import cn.estsh.i3plus.pojo.mes.bean.MesLabelTemplateParam;
import cn.estsh.i3plus.pojo.mes.repository.MesLabelTemplateParamRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesLabelTemplateRepository;
import org.apache.shiro.util.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @Description : 打印模板
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/6/13 17:51
 * @Modify:
 **/
@Service
public class MesTemplateServiceImpl implements IMesTemplateService {

    @Autowired
    private MesLabelTemplateRepository mesLabelTemplateRDao;

    @Autowired
    private MesLabelTemplateParamRepository mesLabelTemplateParamRDao;

    @Override
    public MesLabelTemplate getMesLabelTemplate(String templateCode,String organizeCode) {
        // 根据标签模板代码，查询【MES_标签模板表】，获取对应的方法代码
        MesLabelTemplate labelTemplate = mesLabelTemplateRDao.getByProperty(
                new String[]{MesPcnConstWords.IS_VALID, MesPcnConstWords.IS_DELETED, MesPcnExtConstWords.TEMPLATE_CODE, MesPcnExtConstWords.ORGANIZE_CODE},
                new Object[]{CommonEnumUtil.IS_VAILD.VAILD.getValue(), CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), templateCode,organizeCode});

        if (labelTemplate == null) {
            MesPcnException.throwMesBusiException("标签模板代码【%s】在标签模板表不存在！", templateCode);
        }

        // 根据模板代码查询模板参数信息
        List<MesLabelTemplateParam> templateParamList = mesLabelTemplateParamRDao.findByProperty(new String[]{MesPcnExtConstWords.TEMPLATE_CODE, MesPcnConstWords.IS_VALID, MesPcnConstWords.IS_DELETED,MesPcnExtConstWords.ORGANIZE_CODE},
                new Object[]{templateCode, CommonEnumUtil.IS_VAILD.VAILD.getValue(), CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(),organizeCode});

        if (CollectionUtils.isEmpty(templateParamList)) {
            MesPcnException.throwMesBusiException("标签模板代码【%s】在标签模板明细表不存在！", templateCode);
        }
        labelTemplate.setLabelTemplateParamList(templateParamList);

        return labelTemplate;
    }

    @Override
    public MesLabelTemplate getLabelTemplate(String templateCode, String organizeCode) {
        DdlPackBean templatePackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(templateCode, MesPcnExtConstWords.TEMPLATE_CODE, templatePackBean);
        MesLabelTemplate mesLabelTemplate = mesLabelTemplateRDao.getByProperty(templatePackBean);
        if (mesLabelTemplate != null) {
            //级联获取labelTemplateParam
            DdlPackBean templateParamPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(mesLabelTemplate.getTemplateCode(),  MesPcnExtConstWords.TEMPLATE_CODE, templateParamPackBean);
            List<MesLabelTemplateParam> params = mesLabelTemplateParamRDao.findByHqlWhere(templateParamPackBean);
            mesLabelTemplate.setLabelTemplateParamList(params);
        }
        return mesLabelTemplate;
    }
}
